@@ -43,13 +43,24 @@ struct layout_slot {
 /* Configuration options for the desktop */
 
 int desktop_use_shaped_icons = TRUE;
-int desktop_use_shaped_text = FALSE;
 int desktop_auto_placement = FALSE;
 int desktop_snap_icons = FALSE;
 int desktop_arr_r2l = FALSE;
 int desktop_arr_b2t = FALSE;
 int desktop_arr_rows = FALSE;
 
+/*
+ * Possible values for this one:
+ *
+ * 0 -- Enable shaped text only if window manager is GNOME compliant
+ * 1 -- Enable shaped text always
+ * 2 -- Disable shaped text
+ *
+ * This might seem a bit strange, but it is like that for compatibility reasons.
+ */
+
+int desktop_use_shaped_text = 0;
+
 /* The computed name of the user's desktop directory */
 char *desktop_directory;
 
@@ -661,7 +672,15 @@ desktop_reload_icons (int user_pos, int xpos, int ypos)
 		/* If the file dropped was a .desktop file, pull the suggested
 		 * title and icon from there
 		 */
-		mime = gnome_mime_type_or_default (fau->filename, NULL);
+		if (use_magic)
+		{
+			char *full_name;
+			full_name = g_concat_dir_and_file (desktop_directory, fau->filename);
+			mime = gnome_mime_type_or_default_of_file (full_name, NULL);
+			g_free (full_name);
+		}
+		else
+			mime = gnome_mime_type_or_default (fau->filename, NULL);
 		if (mime && strcmp (mime, "application/x-gnome-app-info") == 0) {
 			GnomeDesktopEntry *entry;
 			char *fullname;
@@ -710,7 +729,7 @@ static WPanel *create_panel_from_desktop(); /* Fwd decl */
 static void free_panel_from_desktop(WPanel *panel);
 
 /* Perform automatic arrangement of the desktop icons */
-static void
+void
 desktop_arrange_icons (SortType type)
 {
 	WPanel *panel;
@@ -2688,6 +2707,9 @@ desktop_popup (GdkEventButton *event)
 
 	/* Create the menu and then strip the tearoff menu items, sigh... */
 
+	if (nodesktop)
+		return;
+
 	popup = gnome_popup_menu_new (desktop_popup_items);
 
 	strip_tearoff_menu_item (desktop_arrange_icons_items);
@@ -3169,11 +3191,13 @@ desktop_init (void)
 {
 	icon_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-	gdnd_init ();
-	gicon_init ();
-	create_layout_info ();
-	create_desktop_dir ();
-	desktop_reload_icons (FALSE, 0, 0);
+	if (!nodesktop) {
+		gdnd_init ();
+		gicon_init ();
+		create_layout_info ();
+		create_desktop_dir ();
+		desktop_reload_icons (FALSE, 0, 0);
+	}
 
 	/* Create the proxy window and initialize all proxying stuff */
 
