@@ -33,6 +33,7 @@
 #include "../vfs/vfs.h"
 #include "gicon.h"
 #include "dialog.h"
+#include "gmain.h"
 
 static void gnome_file_property_dialog_init		(GnomeFilePropertyDialog	 *file_property_dialog);
 static void gnome_file_property_dialog_class_init	(GnomeFilePropertyDialogClass	 *klass);
@@ -198,9 +199,18 @@ create_general_properties (GnomeFilePropertyDialog *fp_dlg)
 	/* File statistics */
 	/* File type first */
   	if (S_ISREG (fp_dlg->st.st_mode)) {
-		gen_string = g_strconcat (_("File Type: "),
-					  gnome_mime_type (fp_dlg->file_name),
-					  NULL);
+		if (use_magic)
+		{
+			gen_string = g_strconcat (_("File Type: "),
+						  gnome_mime_type_or_default_of_file (fp_dlg->file_name, "text/plain"),
+						  NULL);
+		}
+		else
+		{
+			gen_string = g_strconcat (_("File Type: "),
+                                                  gnome_mime_type (fp_dlg->file_name),
+						  NULL);
+		}
 		label = gtk_label_new (gen_string);
 		g_free (gen_string);
 	} else if (S_ISLNK (fp_dlg->st.st_mode)) {
@@ -624,8 +634,9 @@ create_settings_pane (GnomeFilePropertyDialog *fp_dlg)
 }
 
 /* Permissions Pane */
+
 static GtkWidget *
-label_new (char *text, double xalign, double yalign)
+align_label_new (char *text, double xalign, double yalign)
 {
 	GtkWidget *label;
 
@@ -719,15 +730,15 @@ gtk_table_attach (GTK_TABLE (table), widget,		\
 		  GTK_FILL | GTK_SHRINK,		\
 		  0, 0);
 
-#define PERMSET(name, r, w, x, rmask, wmask, xmask, y) do {		\
-	r = perm_check_new (NULL, fp_dlg->st.st_mode & rmask, fp_dlg);			\
-	w = perm_check_new (NULL, fp_dlg->st.st_mode & wmask, fp_dlg);			\
-	x = perm_check_new (NULL, fp_dlg->st.st_mode & xmask, fp_dlg);			\
-									\
-	ATTACH (table, label_new (name, 0.0, 0.5), 0, 1, y, y + 1);	\
-	ATTACH (table, r, 1, 2, y, y + 1);				\
-	ATTACH (table, w, 2, 3, y, y + 1);				\
-	ATTACH (table, x, 3, 4, y, y + 1);				\
+#define PERMSET(name, r, w, x, rmask, wmask, xmask, y) do {			\
+	r = perm_check_new (NULL, fp_dlg->st.st_mode & rmask, fp_dlg);		\
+	w = perm_check_new (NULL, fp_dlg->st.st_mode & wmask, fp_dlg);		\
+	x = perm_check_new (NULL, fp_dlg->st.st_mode & xmask, fp_dlg);		\
+										\
+	ATTACH (table, align_label_new (name, 0.0, 0.5), 0, 1, y, y + 1);	\
+	ATTACH (table, r, 1, 2, y, y + 1);					\
+	ATTACH (table, w, 2, 3, y, y + 1);					\
+	ATTACH (table, x, 3, 4, y, y + 1);					\
 } while (0);
 
 static GtkWidget *
@@ -750,9 +761,10 @@ perm_mode_new (GnomeFilePropertyDialog *fp_dlg)
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
 
-	gtk_box_pack_start (GTK_BOX (hbox), label_new (_("Current mode: "), 0.0, 0.5), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), align_label_new (_("Current mode: "), 0.0, 0.5),
+			    FALSE, FALSE, 0);
 
-	fp_dlg->mode_label = label_new ("0000", 0.0, 0.5);
+	fp_dlg->mode_label = align_label_new ("0000", 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (hbox), fp_dlg->mode_label, FALSE, FALSE, 0);
 
 	table = gtk_table_new (4, 5, FALSE);
@@ -765,10 +777,10 @@ perm_mode_new (GnomeFilePropertyDialog *fp_dlg)
 
 	/* Headings */
 
-	ATTACH (table, label_new (_("Read"), 0.0, 0.5),     1, 2, 0, 1);
-	ATTACH (table, label_new (_("Write"), 0.0, 0.5),    2, 3, 0, 1);
-	ATTACH (table, label_new (_("Exec"), 0.0, 0.5),     3, 4, 0, 1);
-	ATTACH (table, label_new (_("Special"), 0.0, 0.5),  4, 5, 0, 1);
+	ATTACH (table, align_label_new (_("Read"), 0.0, 0.5),     1, 2, 0, 1);
+	ATTACH (table, align_label_new (_("Write"), 0.0, 0.5),    2, 3, 0, 1);
+	ATTACH (table, align_label_new (_("Exec"), 0.0, 0.5),     3, 4, 0, 1);
+	ATTACH (table, align_label_new (_("Special"), 0.0, 0.5),  4, 5, 0, 1);
 
 	/* Permissions */
 
@@ -927,7 +939,7 @@ perm_ownership_new (GnomeFilePropertyDialog *fp_dlg)
 
 	/* Owner */
 
-	gtk_table_attach (GTK_TABLE (table), label_new (_("Owner"), 0.0, 0.5),
+	gtk_table_attach (GTK_TABLE (table), align_label_new (_("Owner"), 0.0, 0.5),
 			  0, 1, 0, 1,
 			  GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK,
 			  0, 0);
@@ -942,7 +954,7 @@ perm_ownership_new (GnomeFilePropertyDialog *fp_dlg)
 
 	/* Group */
 
-	gtk_table_attach (GTK_TABLE (table), label_new (_("Group"), 0.0, 0.5),
+	gtk_table_attach (GTK_TABLE (table), align_label_new (_("Group"), 0.0, 0.5),
 			  0, 1, 1, 2,
 			  GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK,
 			  0, 0);
@@ -1016,7 +1028,13 @@ init_metadata (GnomeFilePropertyDialog *fp_dlg)
 	/*
 	 * Mime type.
 	 */
-	mime_type = (char *) gnome_mime_type_or_default (file_name, NULL);
+	if (use_magic)
+	{
+		mime_type = (char *) gnome_mime_type_or_default_of_file (file_name, NULL);
+	}
+	else
+		mime_type = (char *) gnome_mime_type_or_default (file_name, NULL);
+
 	if (!mime_type)
 		return;
 	fp_dlg->mime_fm_open = gnome_mime_get_value (mime_type, "fm-open");
