@@ -67,9 +67,10 @@ extern int ftpfs_first_cd_then_ls;
 #ifdef HAVE_GNOME
 extern int tree_panel_visible;
 extern int we_can_afford_the_speed;
+int use_magic = 0;
 #endif
 
-/* "$Id: setup.c,v 1.31 1999/09/20 20:03:59 unammx Exp $" */
+/* "$Id: setup.c,v 1.36 2000/02/11 00:33:28 pavel Exp $" */
 
 #ifdef USE_VFS
 extern int vfs_timeout;
@@ -97,7 +98,7 @@ int startup_right_mode;
 /* default panel values */
 int saving_setup;
 
-static struct {
+static const struct {
     char *key;
     sortfn *sort_type;
 } sort_names [] = {
@@ -112,7 +113,7 @@ static struct {
     { 0, 0 }
 };
 
-static struct {
+static const struct {
     char *key;
     int  list_type;
 } list_types [] = {
@@ -124,7 +125,7 @@ static struct {
     { 0, 0 }
 };
 
-static struct {
+static const struct {
     char *opt_name;
     int  opt_type;
 } panel_types [] = {
@@ -135,7 +136,7 @@ static struct {
     { 0, 0 }
 };
 
-static struct {
+static const struct {
     char *opt_name;
     int *opt_addr;
 } layout [] = {
@@ -162,7 +163,7 @@ static struct {
 extern int preserve_uidgid;
 #endif
 
-static struct {
+static const struct {
     char *opt_name;
     int  *opt_addr;
 } options [] = {
@@ -219,6 +220,9 @@ static struct {
     { "xtree_mode", &xtree_mode },
     { "num_history_items_recorded", &num_history_items_recorded },
     { "file_op_compute_totals", &file_op_compute_totals },
+#ifdef HAVE_GNOME
+    { "use_magic", &use_magic },
+#endif
 #ifdef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
     { "dive_into_subdirs", &dive_into_subdirs },
     { "preserve_uidgid", &preserve_uidgid },
@@ -536,7 +540,7 @@ do_load_string (char *s, char *ss, char *def)
 char *
 setup_init (void)
 {
-    static char *buffer;
+    char *buffer;
     char   *profile;
     char   *inifile;
 
