@@ -121,9 +121,9 @@ int altered_magic_flag = 0;
 int altered_nroff_flag = 0;
 /* }}} */
 
-/* "$Id: view.c,v 1.37 1999/06/24 21:53:49 norbert Exp $" */
+/* "$Id: view.c,v 1.39 1999/12/22 04:04:23 proskin Exp $" */
 
-static char hex_char[] = "0123456789ABCDEF";
+static const char hex_char[] = "0123456789ABCDEF";
 
 /* }}} */
 /* {{{ Clean-up functions */
@@ -239,7 +239,12 @@ get_byte (WView *view, unsigned int byte_index)
 		    n = fread (p, 1, VIEW_PAGE_SIZE, view->stdfile);
 		else
 		    n = mc_read (view->file, p, VIEW_PAGE_SIZE);
-#warning FIXME: Errors are ignored at this point, also should report preliminary EOF
+
+/*
+ * FIXME: Errors are ignored at this point
+ * Also should report preliminary EOF
+ */
+
 		if (n != -1)
 		    view->bytes_read += n;
 		if (view->s.st_size < view->bytes_read){
@@ -1362,8 +1367,8 @@ move_left (WView *view)
             if (view->nib_shift == 0)
                 return;
         } 
-        view->edit_cursor = (--view->edit_cursor < view->first) ?
-            view->first : view->edit_cursor;
+        if (view->edit_cursor > view->first)
+	    --view->edit_cursor;
         if (view->edit_cursor < view->start_display) {
            view->edit_cursor += view->bytes_per_line;
            view_move_backward(view, 1);
