@@ -23,7 +23,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-/* "$Id: widget.c,v 1.34 1999/09/14 18:20:03 norbert Exp $" */
+/* "$Id: widget.c,v 1.35 2000/02/03 15:00:12 pavel Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -1748,6 +1748,10 @@ input_new (int y, int x, int color, int len, const char *def_text, char *tkname)
 	    in->history = history_get (tkname);
 	}
     }
+    
+    if (!def_text)
+	def_text="";
+    
     if (def_text == INPUT_LAST_TEXT) {
 	def_text = "";
 	if (in->history)
