@@ -5,6 +5,8 @@
    
    Written by: 1998 Pavel Machek
 
+   $Id: fish.c,v 1.24 2000/02/25 12:53:06 pavel Exp $
+
    Derived from ftpfs.c.
    
    This program is free software; you can redistribute it and/or
@@ -36,6 +38,7 @@
 #undef HAVE_HACKED_SSH
 
 #include "utilvfs.h"
+#include "../src/dialog.h"	/* For MSG_ERROR */
 
 #include "xdirentry.h"
 #include "vfs.h"
@@ -221,12 +224,12 @@ open_archive_int (vfs *me, vfs_s_super *super)
        /dev/tty, not from stdin :-(. */
 
 #ifndef HAVE_HACKED_SSH
-	    message_1s (1, _(" Error "), _("Sorry, we can not do password authenticated connections for now."));
+	    message_1s (1, MSG_ERROR, _("Sorry, we can not do password authenticated connections for now."));
 	    ERRNOR (EPERM, -1);
 #endif
 	    if (!SUP.password){
 		char *p, *op;
-		p = g_strconcat (" fish: Password required for ", SUP.user, 
+		p = g_strconcat (_(" fish: Password required for "), SUP.user, 
 				  " ", NULL);
 		op = vfs_get_password (p);
 		g_free (p);
@@ -470,10 +473,10 @@ file_store(vfs *me, vfs_s_super *super, char *name, char *localname)
 	disable_interrupt_key();
 	total += n;
 	print_vfs_message(_("fish: storing %s %d (%d)"), 
-			  was_error ? "zeros" : "file", total, s.st_size);
+			  was_error ? _("zeros") : _("file"), total, s.st_size);
     }
     if ((get_reply (me, SUP.sockr, NULL, 0) != COMPLETE) || was_error)
-        ERRNOR (E_REMOTE, 0);
+        ERRNOR (E_REMOTE, -1);
     close(h);
     return 0;
 error_return:
