@@ -5,6 +5,8 @@
                1995 Jakub Jelinek
 	       1998 Pavel Machek
    
+   $Id: vfs.c,v 1.54 2000/03/06 16:31:37 pavel Exp $
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License
    as published by the Free Software Foundation; either version 2 of
@@ -114,15 +116,12 @@ static vfs *vfs_list = &vfs_local_ops;
 int
 vfs_register (vfs *vfs)
 {
-    int res;
-
     if (!vfs)
 	vfs_die("You can not register NULL.");
     
-    res = (vfs->init) ? (*vfs->init)(vfs) : 1;
-
-    if (!res)
-	return 0;
+    if (vfs->init)		/* vfs has own initialization function */
+	if (!(*vfs->init)(vfs))	/* but it failed */
+	    return 0;
 
     vfs->next = vfs_list;
     vfs_list = vfs;
@@ -315,7 +314,7 @@ vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
         stamp->id = id;
 	if (parent){
 	    struct vfs_stamping *st = stamp;
-	    for (; parent;){
+	    while (parent){
 		st->parent = g_new (struct vfs_stamping, 1);
 		*st->parent = *parent;
 		parent = parent->parent;
@@ -355,13 +354,12 @@ vfs_stamp (vfs *v, vfsid id)
 void
 vfs_rm_parents (struct vfs_stamping *stamp)
 {
-    struct vfs_stamping *st2, *st3;
+    struct vfs_stamping *parent;
 
-    if (stamp){
-	for (st2 = stamp, st3 = st2->parent; st3 != NULL; st2 = st3, st3 = st3->parent)
-	    g_free (st2);
-
-	g_free (st2);
+    while (stamp) {
+	parent = stamp->parent;
+	g_free (stamp);
+	stamp = parent;
     }
 }
 
@@ -396,16 +394,15 @@ ferrno (vfs *vfs)
 }
 
 int
-mc_open (char *file, int flags, ...)
+mc_open (const char *filename, int flags, ...)
 {
     int  handle;
     int  mode;
     void *info;
-    vfs  *vfs;
     va_list ap;
-    
-    file = vfs_canon (file);
-    vfs = vfs_type (file);
+
+    char *file = vfs_canon (filename);
+    vfs  *vfs  = vfs_type (file);
 
     /* Get the mode flag */	/* FIXME: should look if O_CREAT is present */
     va_start (ap, flags);
@@ -456,12 +453,9 @@ MC_HANDLEOP(read, (int handle, char *buffer, int count), (vfs_info (handle), buf
 int
 mc_ctl (int handle, int ctlop, int arg)
 {
-    vfs *vfs;
-    int result;
+    vfs *vfs = vfs_op (handle);
 
-    vfs = vfs_op (handle);
-    result = vfs->ctl ? (*vfs->ctl)(vfs_info (handle), ctlop, arg) : 0;
-    return result;
+    return vfs->ctl ? (*vfs->ctl)(vfs_info (handle), ctlop, arg) : 0;
 }
 
 int
@@ -692,7 +686,7 @@ off_t mc_lseek (int fd, off_t offset, int whence)
 #define ISSLASH(a) (!a || (a == '/'))
 
 char *
-vfs_canon (char *path)
+vfs_canon (const char *path)
 {
     if (!path)
 	vfs_die("Can not canonize NULL");
@@ -706,18 +700,14 @@ vfs_canon (char *path)
 	    result = vfs_canon (local);
 	    g_free (local);
 	    return result;
-	} else 
-	    return g_strdup (path);
+	}
     }
 
     /* Relative to current directory */
     if (*path != PATH_SEP){ 
     	char *local, *result;
 
-	if (current_dir [strlen (current_dir) - 1] == PATH_SEP)
-	    local = g_strconcat (current_dir, path, NULL);
-	else
-	    local = g_strconcat (current_dir, PATH_SEP_STR, path, NULL);
+	local = concat_dir_and_file (current_dir, path);
 
 	result = vfs_canon (local);
 	g_free (local);
@@ -728,11 +718,15 @@ vfs_canon (char *path)
      * So we have path of following form:
      * /p1/p2#op/.././././p3#op/p4. Good luck.
      */
-    mad_check("(pre-canonicalize)", 0);
-    canonicalize_pathname (path);
-    mad_check("(post-canonicalize)", 0);
+    {
+	char *result = g_strdup (path);
+        
+	mad_check("(pre-canonicalize)", 0);
+	canonicalize_pathname (result);
+	mad_check("(post-canonicalize)", 0);
 
-    return g_strdup (path);
+        return result;
+    }
 }
 
 vfsid
@@ -939,15 +933,20 @@ vfs_current_is_tarfs (void)
 {
     return current_vfs == &vfs_tarfs_ops;
 }
+
+int
+vfs_current_is_cpiofs (void)
+{
+    return current_vfs == &vfs_cpiofs_ops;
+}
 #endif
 
 int
-vfs_file_is_local (char *filename)
+vfs_file_is_local (const char *file)
 {
-    vfs *vfs;
+    char *filename = vfs_canon (file);
+    vfs *vfs = vfs_type (filename);
     
-    filename = vfs_canon (filename);
-    vfs = vfs_type (filename);
     g_free (filename);
     return vfs == &vfs_local_ops;
 }
@@ -967,10 +966,10 @@ vfs_file_is_ftp (char *filename)
 #endif
 }
 
-#ifdef WITH_SMBFS
 int
 vfs_file_is_smb (char *filename)
 {
+#ifdef WITH_SMBFS
 #ifdef USE_NETCODE
     vfs *vfs;
     
@@ -978,17 +977,10 @@ vfs_file_is_smb (char *filename)
     vfs = vfs_type (filename);
     g_free (filename);
     return vfs == &vfs_smbfs_ops;
-#else
+#endif /* USE_NETCODE */
+#endif /* WITH_SMBFS */
     return 0;
-#endif
 }
-#else
-int
-vfs_file_is_smb (char *filename)
-{
-	return 0;
-}
-#endif
 
 char *vfs_get_current_dir (void)
 {
@@ -1003,8 +995,6 @@ static void vfs_setup_wd (void)
 
     if (strlen(current_dir)>MC_MAXPATHLEN-2)
         vfs_die ("Current dir too long.\n");
-
-    return;
 }
 
 MC_NAMEOP (mkdir, (char *path, mode_t mode), (vfs, vfs_name (path), mode))
@@ -1103,18 +1093,17 @@ mc_def_getlocalcopy (vfs *vfs, char *filename)
  fail:
     if (fdout) close(fdout);
     if (fdin) mc_close (fdin);
-    g_free (tmp);
+    free (tmp);
     return NULL;
 }
 
 char *
-mc_getlocalcopy (char *path)
+mc_getlocalcopy (const char *pathname)
 {
-    vfs *vfs;
     char *result;
+    char *path = vfs_canon (pathname);
+    vfs  *vfs  = vfs_type (path);    
 
-    path = vfs_canon (path);
-    vfs = vfs_type (path);    
     result = vfs->getlocalcopy ? (*vfs->getlocalcopy)(vfs, vfs_name (path)) :
                                  mc_def_getlocalcopy (vfs, vfs_name (path));
     g_free (path);
@@ -1157,7 +1146,7 @@ mc_def_ungetlocalcopy (vfs *vfs, char *filename, char *local, int has_changed)
     return 0;
 
  failed:
-    message_1s (1, "Changes to file lost", filename);
+    message_1s (1, _("Changes to file lost"), filename);
     if (fdout) mc_close(fdout);
     if (fdin) close(fdin);
     unlink (local);
@@ -1166,13 +1155,11 @@ mc_def_ungetlocalcopy (vfs *vfs, char *filename, char *local, int has_changed)
 }
 
 int
-mc_ungetlocalcopy (char *path, char *local, int has_changed)
+mc_ungetlocalcopy (const char *pathname, char *local, int has_changed)
 {
-    vfs *vfs;
-    int res;
+    char *path = vfs_canon (pathname);
+    vfs *vfs = vfs_type (path);
 
-    path = vfs_canon (path);
-    vfs = vfs_type (path);
     vfs->ungetlocalcopy ? (*vfs->ungetlocalcopy)(vfs, vfs_name (path), local, has_changed) :
                           mc_def_ungetlocalcopy (vfs, vfs_name (path), local, has_changed);
     /* FIXME: errors are ignored at this point */
@@ -1255,6 +1242,7 @@ vfs_init (void)
     vfs_register (&vfs_extfs_ops);
     vfs_register (&vfs_sfs_ops);
     vfs_register (&vfs_tarfs_ops);
+    vfs_register (&vfs_cpiofs_ops);
 
 #ifdef USE_EXT2FSLIB
     vfs_register (&vfs_undelfs_ops);
@@ -1602,10 +1590,10 @@ int vfs_parse_filedate(int idx, time_t *t)
     
     if (is_num (idx)) {
         if(is_time(columns[idx], &tim) || (got_year = is_year(columns[idx], &tim))) {
-	idx++;
+	    idx++;
 
 	/* This is a special case for ctime() or Mon DD YYYY hh:mm */
-	if(is_num (idx) && 
+	if(is_num (idx) && (columns[idx+1][0]) &&
 	    ((got_year = is_year(columns[idx], &tim)) || is_time(columns[idx], &tim)))
 		idx++; /* time & year or reverse */
 	} /* only time or date */
@@ -1731,8 +1719,11 @@ vfs_parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
         goto error;
     /* Use resulting time value */
     s->st_atime = s->st_ctime = s->st_mtime;
+#if 0
+    /* These variables must be initialized by vfs_s_new_inode () */
     s->st_dev = 0;
     s->st_ino = 0;
+#endif
 #ifdef HAVE_ST_BLKSIZE
     s->st_blksize = 512;
 #endif
@@ -1758,11 +1749,11 @@ vfs_parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 	}
 	if (linkname){
 	    s = g_strdup (p_copy + column_ptr [idx2+1]);
-	    p = strlen (s);
+	    p = strlen (s) - 1;
+	    if (s [p] == '\r' || s [p] == '\n')
+		s [p] = 0;
 	    if (s [p-1] == '\r' || s [p-1] == '\n')
 		s [p-1] = 0;
-	    if (s [p-2] == '\r' || s [p-2] == '\n')
-		s [p-2] = 0;
 		
 	    *linkname = s;
 	}
@@ -1778,12 +1769,12 @@ vfs_parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 	    char *s;
 	    
 	    s = g_strdup (p_copy + column_ptr [idx++]);
-	    p = strlen (s);
+	    p = strlen (s) - 1;
 	    /* g_strchomp(); */
+	    if (s [p] == '\r' || s [p] == '\n')
+	        s [p] = 0;
 	    if (s [p-1] == '\r' || s [p-1] == '\n')
-	        s [p-1] = 0;
-	    if (s [p-2] == '\r' || s [p-2] == '\n')
-		s [p-2] = 0;
+		s [p-1] = 0;
 	    
 	    *filename = s;
 	}
@@ -1798,9 +1789,9 @@ error:
       static int errorcount = 0;
 
       if (++errorcount < 5) {
-	message_1s (1, "Could not parse:", p_copy);
+	message_1s (1, _("Could not parse:"), p_copy);
       } else if (errorcount == 5)
-	message_1s (1, "More parsing errors will be ignored.", "(sorry)" );
+	message_1s (1, _("More parsing errors will be ignored."), _("(sorry)"));
     }
 
     if (p_copy != p)		/* Carefull! */
@@ -1811,7 +1802,7 @@ error:
 void
 vfs_die (char *m)
 {
-    message_1s (1, "Internal error:", m);
+    message_1s (1, _("Internal error:"), m);
     exit (1);
 }
 
